#ifndef _SRAMREADER_H_
#define _SRAMREADER_H_

#ifdef SRAMREADER_EXPORTS
#define SRAMREADER_API __declspec(dllexport)
#else
#define SRAMREADER_API __declspec(dllimport)
#endif

class CSramBank;
class CSramFile;

const int BANK_SIZE = 65536;

//***************************************************************
//                      GetSramReaderVersion
//
// PURPOSE: return the version number of the dll
// PARAMETERS: none
// RETURN VALUE: a pointer on const string with version description
// NOTES:	the string is formated like that : 
//			"vx.x (build configuration)"
//***************************************************************
SRAMREADER_API const char * GetSramReaderVersion();

class SRAMREADER_API CSramDump
{
public:
	//***************************************************************
	//                      CSramDump
	//
	// PURPOSE: construct an empty dump with 4 empty bank
	// PARAMETERS: none
	// RETURN VALUE: none
	// NOTES:	nothing
	//***************************************************************
	CSramDump();
	~CSramDump();

	//***************************************************************
	//                      LoadFromFile
	//
	// PURPOSE: Load a .dump
	// PARAMETERS: pszFileName : name of the file to load
	// RETURN VALUE: return the number of bank loaded, should be 3
	//				for a regular dump.
	// NOTES:	nothing
	//***************************************************************
	int		LoadFromFile(const char * pszFileName);

	//***************************************************************
	//                      SaveToFile
	//
	// PURPOSE: save the dump to a file
	// PARAMETERS: pszFileName : name of the file to write
	// RETURN VALUE: true if success
	// NOTES: return always true for now
	//***************************************************************
	bool	SaveToFile(const char * pszFileName);

	//***************************************************************
	//                      GetBank
	//
	// PURPOSE: get a pointer on one of the 3 last bank
	// PARAMETERS: nBank : index of the bank (1,2 or 3)
	// RETURN VALUE: pointer on a bank
	// NOTES: the pointer returned can't be NULL
	//***************************************************************
	CSramBank	*	GetBank(int nBank);

	//***************************************************************
	//                      GetWorkingBank
	//
	// PURPOSE: get a pointer on the data contain in the working bank
	// PARAMETERS: none
	// RETURN VALUE: pointer on a 64ko buffer
	// NOTES: nothing
	//***************************************************************
	void		*	GetWorkingBank();

private:
	CSramBank	*	m_pSramBank[3];
	void		*	m_pWorkingBank;
};

class SRAMREADER_API CSramBank
{
public:
	CSramBank();
	~CSramBank();

	//***************************************************************
	//                      LoadFromFile
	//
	// PURPOSE: Load Sram file system from a buffer
	// PARAMETERS: pBuffer : buffer that contain the file system
	// RETURN VALUE: return true if success
	// NOTES: - the buffer must start by MINSF2 to be recognize
	//			as a  valid Sram file system.
	//		  - all the file contain in the bank will be erase
	//***************************************************************
	bool LoadFromBuffer(char * pBuffer);

	//***************************************************************
	//                      SaveToBuffer
	//
	// PURPOSE: save the Sram file system to a buffer
	// PARAMETERS:	pBuffer : buffer to the file system in
	//				nBufferSize : size of the buffer
	// RETURN VALUE: return true if success, false if the buffer is
	//				the buffer is too small
	// NOTES:	the buffer should be 65536 bytes to be sure that 
	//			everything will fit
	//***************************************************************
	bool SaveToBuffer(char * pBuffer,int nBufferSize);

	//***************************************************************
	//                      GetNumFile
	//
	// PURPOSE: return the number of files in the bank
	// PARAMETERS:	none
	// RETURN VALUE: number of file in the bank
	// NOTES:	nothing
	//***************************************************************
	int	GetNumFile();
	
	//***************************************************************
	//                      GetFirstFile
	//
	// PURPOSE: return the first file of the bank
	// PARAMETERS:	none
	// RETURN VALUE: pointer on the first file in the bank, NULL if
	//				the bank is empty
	// NOTES:	nothing
	//***************************************************************
	CSramFile * GetFirstFile();

	//***************************************************************
	//                      GetNextFile
	//
	// PURPOSE: return the next file in the bank
	// PARAMETERS:	pSramFile : pointer on current file
	// RETURN VALUE: pointer on the file after pSramFile in the bank,
	//				return NULL if pSramFile was the last file
	// NOTES:	pSramFile must belong to this bank
	//***************************************************************
	CSramFile * GetNextFile(CSramFile * pSramFile);
	
	//***************************************************************
	//                    GetBankUsedSpace
	//
	// PURPOSE: return space used by the bank in byte
	// PARAMETERS:	none
	// RETURN VALUE: space used by the file system in the bank
	// NOTES:	nothing
	//***************************************************************
	int	GetBankUsedSpace();
	
	//***************************************************************
	//                    GetBankFreeSpace
	//
	// PURPOSE: return space free in the bank in byte
	// PARAMETERS:	none
	// RETURN VALUE: space free in the bank
	// NOTES:	return BANK_SIZE - GetBankUsedSpace()
	//***************************************************************
	int	GetBankFreeSpace();

	//***************************************************************
	//						ExtractFile
	//
	// PURPOSE: extract an Sram file
	// PARAMETERS:	pSramFile : the Sram file to extract
	//				pszDestFile : the name of the file to write in
	// RETURN VALUE: none
	// NOTES:	pSramFile must belong to this bank
	//***************************************************************
	void ExtractFile(CSramFile * pSramFile,const char * pszDestFile);

	//***************************************************************
	//							AddFile
	//
	// PURPOSE: Add a file to the bank
	// PARAMETERS:	pszFileName : name of the file to add
	//				pszSramFileName : name that will have the file
	//					in the bank
	//				pszramHeader : header of the savegame : must be
	//					8 bytes long	
	// RETURN VALUE: return NULL when there is not enough space to 
	//				add the file, a pointer on the new file otherwise
	// NOTES: - you can only add savegame with function
	//		  - Assert if a file with the same name already exist,
	//			use FindByName to handle it yourself
	//		  - Assert if a file with the same header already exist
	//			use FindByHeader to handle it yourself
	//		  - the header must be 'S''A''V''E''\0' + gamecode or
	//		  	'S''A''V''E' + crc
	//***************************************************************
	CSramFile * AddFile(const char * pszFileName,const char * pszSramFileName,const unsigned char * pszSramHeader);

	//***************************************************************
	//							AddFile
	//
	// PURPOSE: Add a file to the bank from a SramFile pointer
	// PARAMETERS:	pSramFile : pointer on the file to add
	// RETURN VALUE: return NULL when there is not enough space to 
	//				add the file, a pointer on the new file otherwise
	// NOTES: - the file will be duplicated, so you can erase the 
	//			original afterward
	//		  - The SramFile pointer can came from another bank
	//		  - Assert if a file with the same name already exist,
	//			use FindByName to handle it yourself
	//		  - Assert if a file with the same header already exist
	//			use FindByHeader to handle it yourself
	//***************************************************************
	CSramFile * AddFile(CSramFile * pSramFile);

	//***************************************************************
	//						ImportState
	//
	// PURPOSE: import a .state (pogo config) into the bank
	// PARAMETERS:	pszFileName : name of the .state
	// RETURN VALUE: return NULL when there is not enough space to 
	//				add the .state, a pointer on the new file otherwise
	// NOTES: you should only add .state to bank 1
	//***************************************************************
	CSramFile * ImportState(const char * pszFileName);

	//***************************************************************
	//						RemoveFile
	//
	// PURPOSE: remove a file from the bank
	// PARAMETERS:	pSramFile : the Sram file to remove
	// RETURN VALUE: none
	// NOTES:	pSramFile must belong to this bank
	//***************************************************************
	void RemoveFile(CSramFile * pSramFile);
	
	//***************************************************************
	//						FindByName
	//
	// PURPOSE: Check if a file with the same name already exist
	// PARAMETERS:	pszName : name to search for
	//				pSearchAfter : file after you which you want 
	//					to start the search, NULL if you want to
	//					start from the first file
	// RETURN VALUE: return a pointer to the file with that name,
	//				NULL if none are found
	// NOTES:	you can use this function to check for duplicate
	//			name
	//***************************************************************
	CSramFile *  FindByName(const char * pszName,CSramFile * pSearchAfter = NULL);
	
	//***************************************************************
	//						FindByHeader
	//
	// PURPOSE: Check if a file with the same header already exist
	// PARAMETERS:	pszHeader : header to search for, must be
	//					8 bytes long
	//				pSearchAfter : file after you which you want 
	//					to start the search, NULL if you want to
	//					start from the first file
	// RETURN VALUE: return a pointer to the file with that header,
	//				NULL if none are found
	// NOTES: - you can use this function to check for duplicate
	//			name
	//***************************************************************
	CSramFile *  FindByHeader(const unsigned char * pszHeader,CSramFile * pSearchAfter = NULL);

private:
	//those two struct match the memory in the gba Sram
	struct SPogoSramFile
	{
		unsigned long	m_nNext;
		char			m_pszName[32];
		unsigned short	m_nFlags ;
		unsigned short	m_nLowWordLength;	//I used a 2 short cause of memory alignment
		unsigned short	m_nHighWordLength;	//issue, please use GetLenght()
		char			m_DataStart;		//just convenient to get the address of the data

		unsigned long	GetLenght();
		void			SetLenght(unsigned long nLength);
	};

	struct SPogoBankHeader
	{
		char			m_pszIdent[8];
		unsigned long	m_nFirstPtr;
		unsigned long	m_nGapPtr;
	};
	int			m_nNumFile;
	CSramFile * m_pFileList;

	CSramFile * AddFileFromDump(SPogoSramFile * pSramFile,bool bAddTail = false);
	//return NULL when there is not enough free space in the bank to add the file
	CSramFile * AddFileFromBuffer(const char * pszName, const unsigned char * pBuffer,unsigned long nBufferLen,bool bAddTail = false);
	void AddNode(CSramFile * pNode,bool bAddTail);

	//dest buffer should be at least 64ko long
	void	RleUnpack(unsigned char * pSrc, unsigned char * pDest, int nSrcSize);
	int		RlePack(unsigned char * pSrc, unsigned char * pDest, int nSize);
};

class SRAMREADER_API CSramFile
{
public:
	CSramFile();
	~CSramFile();

	const char		*	GetName();
	unsigned short		GetFlags();
	unsigned long		GetDataSize();
	bool				IsSavegame();
	bool				HasGameCodeID();
	unsigned char	*	GetData();
	bool				Rename(const char * pszName);

private:
	friend class CSramBank;

	bool				IsValid();

	unsigned long		m_nSecurityCheck;
	char				m_pszName[32];
	unsigned short		m_nFlags;
	unsigned long		m_nLength; 
	bool				m_bSavegame;
	bool				m_bGameCodeMode;
	unsigned char	*	m_pData;
	CSramFile		*	m_pNext;
	CSramFile		*	m_pPrev;
	void			*	m_pOwner;
};

#endif //_SRAMREADER_H_